import { defineStore } from 'pinia'
import * as api from '@/api'
import { useWorkerStore } from '@/stores'
import { UUID } from '@/types'
import { Repository, WorkerLight } from '@/types/worker'

interface State {
  /**
   * Repository details mapped by ID
   */
  repositories: {
    [repositoryId: UUID]: Repository
  }
}

export const useRepositoryStore = defineStore('repos', {
  state: (): State => ({
    repositories: {}
  }),
  actions: {
    async retrieveRepository (id: UUID) {
      const repo = await api.retrieveRepository(id)
      this.repositories[repo.id] = repo
    },
    async listRepository (page = 1) {
      // List repositories imported on Arkindex
      const data = await api.listRepositories({ page })
      this.repositories = {
        ...this.repositories,
        ...Object.fromEntries(data.results.map((repo: Repository) => [repo.id, repo]))
      }
      // Set workers in the appropriate store to retrieve them following a link later
      const workerStore = useWorkerStore()
      const workers = data.results.flatMap(({ workers }) => workers)
      workerStore.workers = {
        ...workerStore.workers,
        ...Object.fromEntries(workers.map((worker: WorkerLight) => [worker.id, worker]))
      }
      return data
    },
    async deleteRepository (id: UUID) {
      const data = await api.deleteRepository(id)
      delete this.repositories[id]
      return data
    }
  }
})
