import { assert } from 'chai'
import { createPinia, setActivePinia } from 'pinia'
import { useNotificationStore } from '@/stores'
import store from './index.spec'

describe('notifications', () => {
  let notificationStore

  beforeEach(() => {
    setActivePinia(createPinia())
    notificationStore = useNotificationStore()
    store.reset()
  })

  describe('mutations', () => {
    describe('notify', () => {
      it('forwards to Pinia', () => {
        // No state inside Vuex
        assert.deepStrictEqual(store.state.notifications, {})
        // The state in Pinia should be empty
        assert.deepStrictEqual(notificationStore.notifications, [])

        store.commit('notifications/notify', { type: 'error', text: 'Something' })

        // Still nothing in Vuex
        assert.deepStrictEqual(store.state.notifications, {})
        // There is a notification in the Pinia store
        assert.deepStrictEqual(notificationStore.notifications, [
          {
            id: 0,
            type: 'error',
            text: 'Something'
          }
        ])
      })
    })

    describe('reset', () => {
      it('forwards to Pinia', () => {
        notificationStore.notifications = [
          {
            id: 0,
            type: 'error',
            text: 'Something'
          }
        ]

        store.commit('notifications/reset')

        // No state inside Vuex
        assert.deepStrictEqual(store.state.notifications, {})
        // The state in Pinia should be empty
        assert.deepStrictEqual(notificationStore.notifications, [])
      })
    })
  })
})
